//Written for www.HomoFaciens.de
//Program calculates energy and power needed to drive a car.
//Copyright (C) 2010 Norbert Heinz
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free //Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or //FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.image.*;
import java.io.*;
import java.net.URL;
import javax.imageio.ImageIO;
import javax.swing.*;
import java.text.*;
import java.util.GregorianCalendar;
import java.util.Calendar;
import javax.sound.sampled.*;
import java.applet.*;
import java.util.Hashtable;
import javax.swing.event.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;




public class mechanicalperformance extends JApplet implements  ActionListener, MouseListener  {

    static String versionNumber="v1.0";

    ScenePainter painter;
    GraphPainter graph01;
    JScrollPane textScrollPane;
    JTextArea trackTextArea;
    JSpinner powerSpinner, cwSpinner, crSpinner, massSpinner, areaSpinner;
    JSlider throttlePedalSlider;
    JCheckBox carPowerCheckBox, airPowerCheckBox, rollPowerCheckBox, kineticPowerCheckBox, potentialPowerCheckBox, fixThrottleCheckBox;
    SpinnerNumberModel powerModel, cwModel, crModel, massModel, areaModel;
    JButton calcButton, pauseButton, editTrackButton;
    JLabel powerLabel, cwLabel, crLabel, massLabel, areaLabel, throttlePedalLabel;
    JLabel powerUnitLabel, massUnitLabel, areaUnitLabel;
    JLabel sourceLabel, versionLabel;
    double coordsCount=350.0; //number of coordinates to calculate for graphpainting
    boolean isRunning=false; //is simulation still running?
    Timer timer;             //timer for simulation
    int timerPause=25;       //delay between two calculated frames in milliseconds
    long simulationTime, startTime, simulationTimeOld;  //times passed while simulation is running
    double coordMulti=0.0;
    static String language="initalValue";
    String givenParam="nothing yet";   //Arguments passed to applet
    double doveSpeed=15.0;
    double doveCycle;         //Wingposition while dove is flying
    String speedName, timeName, accelerationName, startName, stopName, minName, maxName, breakName, throttlePedalName;
    String powerName, areaName;
    String cwName, crName, massName;
    String hint01Name, hint02Name;
    String truckImageName, carImageName, guidePostImageName, backGroundImageName, sunImageName, moonImageName;
    String carTireImageName, ImageName;
    String carPowerName, airPowerName, rollPowerName, kineticPowerName, potentialPowerName;
    String signalGreenImageName, signalYellowImageName, signalRedImageName, carTireDrivingImageName;
    String doveWingImageName, doveImageName, fixThrottleName, gradientSign01Name, gradientSign02Name;
    String doveSoundWaveName, carSoundWaveName, tireSquealingWaveName, averageSpeedName, energyInputName;;
    String pauseName, noPauseName, editTrackName, editTrackDoneName, editTrackHintName, trackHeightName;
    boolean editMode=false;
    int lapCounter=0;

    Font font = new Font("Arial", Font.PLAIN, 12);
    Font fontFixed = new Font("Monospaced", 0, 12);

    public void init() {
        UIManager.put("swing.boldMetal", Boolean.FALSE);

        try{
          givenParam = getParameter("LanguageSettings");
        }
        catch(NullPointerException npe){
          System.out.println("could not get parameter 'LanguageSettings'");
        }

        if(givenParam.compareTo("nothing yet")!=0) {
          if (givenParam.compareTo("German")==0)language="German";
          else language="English";
        }
    }

    public void start() {

        initComponents();

        timer = new Timer(timerPause, new timerEvent());
    }

    public static void main(String[] args) {
        String appName="Mechanical performance";

        if(args.length>0){
          if (args[0].compareTo("German")==0){
            language="German";
            appName="Mechanische Leistung";
          }
          else{
            language="English";
            appName="Mechanical performance";
          }
        }
        else{
          language="English";
          appName="Mechanical performance";
        }

        JFrame f = new JFrame(appName);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JApplet ap = new mechanicalperformance();
        ap.init();
        ap.start();
        f.getContentPane().add("Center", ap);
        f.pack();
        f.setVisible(true);

    }


    private BufferedImage loadImage(String name) {
        String imgFileName = "images/"+name;
        URL url = mechanicalperformance.class.getResource(imgFileName);
        BufferedImage img = null;
        try {
            img =  ImageIO.read(url);
        }
        catch (Exception e){
          img=null;
          System.out.println("Error loading Image '" + imgFileName + "': " + e); // Display the string.
        }
        return img;
    }

    private AudioClip loadSound(String name) {
        String soundFileName = "sounds/"+name;
        AudioClip audioClp=null;
        try {
          URL url = this.getClass().getClassLoader().getResource(soundFileName);
          audioClp = newAudioClip(url);
        }
        catch (Exception e){
          audioClp=null;
          System.out.println("Error loading Sound '" + soundFileName + "': " + e); // Display the string.
        }
        return audioClp;
    }

    public void setSpinnerWidth() {
        JFormattedTextField ftf = null;
        ftf = getTextField(powerSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(6); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
        ftf = getTextField(cwSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(6); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
        ftf = getTextField(crSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(6); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
        ftf = getTextField(massSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(6); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
        ftf = getTextField(areaSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(6); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
    }

    public void initComponents() {
        int i;
        GridBagConstraints c = new GridBagConstraints();

        getContentPane().setLayout(new BorderLayout());

        //create panels
        JPanel parentControlPanel = new JPanel();
        JPanel checkBoxesControlPanel = new JPanel();
        JPanel controlPanel = new JPanel();
        JPanel scenePanel = new JPanel();
        JPanel mainPane = new JPanel();
        JPanel graphPanel = new JPanel();
        JPanel throttlePedalPanel = new JPanel();

        //set panel properties
        controlPanel.setLayout(new GridBagLayout());
        throttlePedalPanel.setLayout(new GridBagLayout());
        parentControlPanel.setLayout(new BoxLayout(parentControlPanel, BoxLayout.PAGE_AXIS));
        mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
        mainPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        if(language.compareTo("German")==0){
          timeName="Zeit";
          powerName="Leistung";
          cwName="CW-Wert";
          crName="CR-Wert";
          massName="Masse";
          areaName="Stirnfläche";
          speedName="Geschwindigkeit";
          accelerationName="Beschleunigung";
          carPowerName="Motor";
          airPowerName="Luft";
          rollPowerName="Roll";
          kineticPowerName="Kin.";
          potentialPowerName="Pot.";
          fixThrottleName="Gas fixieren";
          startName="Start";
          stopName="Stop";
          pauseName="Pause";
          noPauseName="Weiter";
          minName="Kein Gas";
          maxName="Max";
          breakName="Bremsen";
          editTrackName="Streckenführung";
          editTrackDoneName="Daten übernehmen";
          editTrackHintName="Jeder Streckenabschnitt ist durch seine Nummer in Klammern '()' gekennzeichnet.\n" + 
                            "Die erste Zahl gibt die Abschnittslänge, die Zweite die Steigung an.\n" + 
                            "Nur ganzzahlige Werte werden akzeptiert (0 bis 10000 für Länge, -20 bis 20 für Steigung)!\n" +
                            "Bis zu 100 Streckenabschnitte können definiert werden." + 
                            "Die Bearbeitung endet an dem ersten Streckenabschnitt mit der Länge = 0!!!";
          throttlePedalName="Gaspedal";
          hint01Name="Verschiebe mit der Maus das Gaspedal zum Beschleunigen !!!";
          hint02Name="Durchdrehende Reifen verhindern optimales Beschleunigen !!!";

          carImageName = "fiat126.png";
          carTireImageName = "fiat126-tire.png";
          carTireDrivingImageName = "fiat126-tire-driving.png";
          guidePostImageName = "guidepost.png";
          backGroundImageName = "background.png";
          sunImageName = "sun.png";
          moonImageName = "moon.png";
          doveImageName = "dove.png";
          doveWingImageName = "dovewing.png";
          signalRedImageName = "signal_light_red.png";
          signalYellowImageName = "signal_light_yellow.png";
          signalGreenImageName = "signal_light_green.png";
          gradientSign01Name = "gradient01.png";
          gradientSign02Name = "gradient02.png";
          averageSpeedName= "Durchscnittsgeschwindigkeit";
          trackHeightName = "Höhe über Null";
          energyInputName = "Energieverbrauch";

          carSoundWaveName = "car.wav";
          doveSoundWaveName = "dove.wav";
          tireSquealingWaveName = "squealing.wav";

        }
        else if(language.compareTo("XXX")==0){
          timeName="XXX";
        }
        else{
          timeName="Time";
          powerName="Power";
          cwName="Cd-value";
          crName="Crr-value";
          massName="Mass";
          areaName="Reference area";
          speedName="Speed";
          accelerationName="Acceleration";
          carPowerName="Engine";
          airPowerName="Air";
          rollPowerName="Roll";
          kineticPowerName="Kin.";
          potentialPowerName="Pot.";
          fixThrottleName="Fix throttle";
          startName="Start";
          stopName="Stop";
          pauseName="Pause";
          noPauseName="Resume";
          minName="No throttle";
          maxName="Max";
          breakName="Brake";
          editTrackName="Edit track";
          editTrackDoneName="Use new track";
          editTrackHintName="Every track section is marked by its number in brakets '()'.\n" + 
                            "The first value stands for the section length, the second for the gradient, seperated by '/'.\n" + 
                            "Only integer values are accepted (0 to 10000 for lenght, -20 to 20 for gradient!\n" +
                            "Up to 100 sections can be defined.\n" + 
                            "Computing stops at the first section found with length = 0!!!";
          throttlePedalName="Throttle pedal";
          hint01Name="Move throttle pedal with your mouse to accelerate !!!";
          hint02Name="No burnouts for optimal acceleration !!!";

          carImageName = "fiat126.png";
          carTireImageName = "fiat126-tire.png";
          carTireDrivingImageName = "fiat126-tire-driving.png";
          guidePostImageName = "guidepost.png";
          backGroundImageName = "background_en.png";
          sunImageName = "sun.png";
          moonImageName = "moon.png";
          doveImageName = "dove.png";
          doveWingImageName = "dovewing.png";
          signalRedImageName = "signal_light_red.png";
          signalYellowImageName = "signal_light_yellow.png";
          signalGreenImageName = "signal_light_green.png";
          gradientSign01Name = "gradient01.png";
          gradientSign02Name = "gradient02.png";
          averageSpeedName = "Average Speed";
          trackHeightName = "Level above zero";
          energyInputName = "Energy used";

          carSoundWaveName = "car.wav";
          doveSoundWaveName = "dove.wav";
          tireSquealingWaveName = "squealing.wav";

        }

        //create buttons, labels etcetera

        throttlePedalSlider = new JSlider(JSlider.VERTICAL, 0, 1200, 0);

        calcButton = new JButton(startName);
        pauseButton = new JButton(pauseName);
        editTrackButton = new JButton(editTrackName);

        carPowerCheckBox = new JCheckBox(carPowerName);
        airPowerCheckBox = new JCheckBox(airPowerName);
        rollPowerCheckBox = new JCheckBox(rollPowerName);
        kineticPowerCheckBox = new JCheckBox(kineticPowerName);
        potentialPowerCheckBox = new JCheckBox(potentialPowerName);
        fixThrottleCheckBox = new JCheckBox(fixThrottleName);

        powerLabel = new JLabel(powerName + ":");
        cwLabel = new JLabel(cwName + ":");
        crLabel = new JLabel(crName + ":");
        massLabel = new JLabel(massName + ":");
        areaLabel = new JLabel(areaName + ":");

        throttlePedalLabel = new JLabel(throttlePedalName);

        powerUnitLabel = new JLabel("kW");
        massUnitLabel = new JLabel("kg");
        areaUnitLabel = new JLabel("m²");

        sourceLabel = new JLabel("Source & info:    www.HomoFaciens.de");
        versionLabel = new JLabel(versionNumber);

        powerModel = new SpinnerNumberModel(17 , 1, 300, 1);
        cwModel = new SpinnerNumberModel(0.47 , 0.0, 1.50, 0.01);
        crModel = new SpinnerNumberModel(0.020 , 0.0, 0.200, 0.001);
        massModel = new SpinnerNumberModel(765 , 1, 1000, 1);
        areaModel = new SpinnerNumberModel(1.51 , 0.1, 10.0, 0.01);

        powerSpinner = new JSpinner(powerModel);
        cwSpinner = new JSpinner(cwModel);
        crSpinner = new JSpinner(crModel);
        massSpinner = new JSpinner(massModel);
        areaSpinner = new JSpinner(areaModel);
        setSpinnerWidth();

        painter = new ScenePainter();
        graph01 = new GraphPainter();

        //set properties of controls
        carPowerCheckBox.setFont(font);
        carPowerCheckBox.setSelected(true);
        airPowerCheckBox.setFont(font);
        airPowerCheckBox.setSelected(true);
        rollPowerCheckBox.setFont(font);
        rollPowerCheckBox.setSelected(true);
        kineticPowerCheckBox.setFont(font);
        kineticPowerCheckBox.setSelected(true);
        potentialPowerCheckBox.setFont(font);
        potentialPowerCheckBox.setSelected(true);
        fixThrottleCheckBox.setFont(font);
        fixThrottleCheckBox.setSelected(false);

        graph01.xString="s [m]";
        graph01.yString="P [kW]";

        painter.hint01Name=hint01Name;
        painter.hint02Name=hint02Name;
        painter.averageSpeedName=averageSpeedName;
        painter.trackHeightName=trackHeightName;
        painter.energyInputName = energyInputName;

        for(i=0;i<100;i++){
          painter.trackShapeDistance[i]=0;
          painter.trackShapeGradient[i]=0;
          painter.trackShapeHeightSum[i]=0.0;
        }
        for(i=0;i<100;i+=2){
          painter.trackShapeGradient[i]=0;
        }

        painter.trackShapeDistance[0]=2000;
        painter.trackShapeGradient[0]=0;
        painter.trackShapeDistance[1]=2000;
        painter.trackShapeGradient[1]=5;
        painter.trackShapeDistance[2]=2000;
        painter.trackShapeGradient[2]=0;
        painter.trackShapeDistance[3]=2000;
        painter.trackShapeGradient[3]=-5;
        painter.trackShapeDistance[4]=1763;
        painter.trackShapeGradient[4]=0;
        //starting hole
        painter.trackShapeDistance[5]=3;
        painter.trackShapeGradient[5]=-1;
        painter.trackShapeDistance[6]=3;
        painter.trackShapeGradient[6]=-2;
        painter.trackShapeDistance[7]=3;
        painter.trackShapeGradient[7]=-3;
        painter.trackShapeDistance[8]=3;
        painter.trackShapeGradient[8]=-4;
        painter.trackShapeDistance[9]=3;
        painter.trackShapeGradient[9]=-5;
        painter.trackShapeDistance[10]=3;
        painter.trackShapeGradient[10]=-6;
        painter.trackShapeDistance[11]=3;
        painter.trackShapeGradient[11]=-7;
        painter.trackShapeDistance[12]=3;
        painter.trackShapeGradient[12]=-8;
        painter.trackShapeDistance[13]=3;
        painter.trackShapeGradient[13]=-9;
        painter.trackShapeDistance[14]=3;
        painter.trackShapeGradient[14]=-10;
        painter.trackShapeDistance[15]=3;
        painter.trackShapeGradient[15]=-9;
        painter.trackShapeDistance[16]=3;
        painter.trackShapeGradient[16]=-8;
        painter.trackShapeDistance[17]=3;
        painter.trackShapeGradient[17]=-7;
        painter.trackShapeDistance[18]=3;
        painter.trackShapeGradient[18]=-6;
        painter.trackShapeDistance[19]=3;
        painter.trackShapeGradient[19]=-5;
        painter.trackShapeDistance[20]=3;
        painter.trackShapeGradient[20]=-4;
        painter.trackShapeDistance[21]=3;
        painter.trackShapeGradient[21]=-3;
        painter.trackShapeDistance[22]=3;
        painter.trackShapeGradient[22]=-2;
        painter.trackShapeDistance[23]=3;
        painter.trackShapeGradient[23]=-1;
        painter.trackShapeDistance[24]=3;
        painter.trackShapeGradient[24]=0;
        painter.trackShapeDistance[25]=3;
        painter.trackShapeGradient[25]=1;
        painter.trackShapeDistance[26]=3;
        painter.trackShapeGradient[26]=2;
        painter.trackShapeDistance[27]=3;
        painter.trackShapeGradient[27]=3;
        painter.trackShapeDistance[28]=3;
        painter.trackShapeGradient[28]=4;
        painter.trackShapeDistance[29]=3;
        painter.trackShapeGradient[29]=5;
        painter.trackShapeDistance[30]=3;
        painter.trackShapeGradient[30]=6;
        painter.trackShapeDistance[31]=3;
        painter.trackShapeGradient[31]=7;
        painter.trackShapeDistance[32]=3;
        painter.trackShapeGradient[32]=8;
        painter.trackShapeDistance[33]=3;
        painter.trackShapeGradient[33]=9;
        painter.trackShapeDistance[34]=3;
        painter.trackShapeGradient[34]=10;
        painter.trackShapeDistance[35]=3;
        painter.trackShapeGradient[35]=9;
        painter.trackShapeDistance[36]=3;
        painter.trackShapeGradient[36]=8;
        painter.trackShapeDistance[37]=3;
        painter.trackShapeGradient[37]=7;
        painter.trackShapeDistance[38]=3;
        painter.trackShapeGradient[38]=6;
        painter.trackShapeDistance[39]=3;
        painter.trackShapeGradient[39]=5;
        painter.trackShapeDistance[40]=3;
        painter.trackShapeGradient[40]=4;
        painter.trackShapeDistance[41]=3;
        painter.trackShapeGradient[41]=3;
        painter.trackShapeDistance[42]=3;
        painter.trackShapeGradient[42]=2;
        painter.trackShapeDistance[43]=3;
        painter.trackShapeGradient[43]=1;
        painter.trackShapeDistance[44]=3;
        painter.trackShapeGradient[44]=0;
        //starting hill
        painter.trackShapeDistance[45]=3;
        painter.trackShapeGradient[45]=1;
        painter.trackShapeDistance[46]=3;
        painter.trackShapeGradient[46]=2;
        painter.trackShapeDistance[47]=3;
        painter.trackShapeGradient[47]=3;
        painter.trackShapeDistance[48]=3;
        painter.trackShapeGradient[48]=4;
        painter.trackShapeDistance[49]=3;
        painter.trackShapeGradient[49]=5;
        painter.trackShapeDistance[50]=3;
        painter.trackShapeGradient[50]=6;
        painter.trackShapeDistance[51]=3;
        painter.trackShapeGradient[51]=7;
        painter.trackShapeDistance[52]=3;
        painter.trackShapeGradient[52]=8;
        painter.trackShapeDistance[53]=3;
        painter.trackShapeGradient[53]=9;
        painter.trackShapeDistance[54]=3;
        painter.trackShapeGradient[54]=10;
        painter.trackShapeDistance[55]=3;
        painter.trackShapeGradient[55]=9;
        painter.trackShapeDistance[56]=3;
        painter.trackShapeGradient[56]=8;
        painter.trackShapeDistance[57]=3;
        painter.trackShapeGradient[57]=7;
        painter.trackShapeDistance[58]=3;
        painter.trackShapeGradient[58]=6;
        painter.trackShapeDistance[59]=3;
        painter.trackShapeGradient[59]=5;
        painter.trackShapeDistance[60]=3;
        painter.trackShapeGradient[60]=4;
        painter.trackShapeDistance[61]=3;
        painter.trackShapeGradient[61]=3;
        painter.trackShapeDistance[62]=3;
        painter.trackShapeGradient[62]=2;
        painter.trackShapeDistance[63]=3;
        painter.trackShapeGradient[63]=1;
        painter.trackShapeDistance[64]=3;
        painter.trackShapeGradient[64]=0;
        painter.trackShapeDistance[65]=3;
        painter.trackShapeGradient[65]=-1;
        painter.trackShapeDistance[66]=3;
        painter.trackShapeGradient[66]=-2;
        painter.trackShapeDistance[67]=3;
        painter.trackShapeGradient[67]=-3;
        painter.trackShapeDistance[68]=3;
        painter.trackShapeGradient[68]=-4;
        painter.trackShapeDistance[69]=3;
        painter.trackShapeGradient[69]=-5;
        painter.trackShapeDistance[70]=3;
        painter.trackShapeGradient[70]=-6;
        painter.trackShapeDistance[71]=3;
        painter.trackShapeGradient[71]=-7;
        painter.trackShapeDistance[72]=3;
        painter.trackShapeGradient[72]=-8;
        painter.trackShapeDistance[73]=3;
        painter.trackShapeGradient[73]=-9;
        painter.trackShapeDistance[74]=3;
        painter.trackShapeGradient[74]=-10;
        painter.trackShapeDistance[75]=3;
        painter.trackShapeGradient[75]=-9;
        painter.trackShapeDistance[76]=3;
        painter.trackShapeGradient[76]=-8;
        painter.trackShapeDistance[77]=3;
        painter.trackShapeGradient[77]=-7;
        painter.trackShapeDistance[78]=3;
        painter.trackShapeGradient[78]=-6;
        painter.trackShapeDistance[79]=3;
        painter.trackShapeGradient[79]=-5;
        painter.trackShapeDistance[80]=3;
        painter.trackShapeGradient[80]=-4;
        painter.trackShapeDistance[81]=3;
        painter.trackShapeGradient[81]=-3;
        painter.trackShapeDistance[82]=3;
        painter.trackShapeGradient[82]=-2;
        painter.trackShapeDistance[83]=3;
        painter.trackShapeGradient[83]=-1;

        painter.trackDistance=0;
        for(i=0;i<100;i++){
          painter.trackDistance += painter.trackShapeDistance[i];
          if(i>0){
            painter.trackShapeHeightSum[i]=painter.trackShapeHeightSum[i-1] + 
                                           painter.trackShapeDistance[i] * Math.sin(Math.atan(painter.trackShapeGradient[i] / 100.0));
          }
          else{
            painter.trackShapeHeightSum[i]=painter.trackShapeDistance[i] * Math.sin(Math.atan(painter.trackShapeGradient[i] / 100.0));
          }
          if(painter.trackShapeDistance[i]==0){
            painter.trackIMax=i+1;
            break;
          }
        }

        //Hashtable<Integer, JLabel> throttleLabelTable =  new Hashtable<Integer, JLabel>();//Java 1.5 way to define Hashtable in a more save way
        Hashtable throttleLabelTable =  new Hashtable();//Java 1.4 (for old win98 operating system - I still like it)
        throttleLabelTable.put(new Integer( 0 ), new JLabel(breakName) );
        throttleLabelTable.put(new Integer( 200 ), new JLabel(minName) );
        throttleLabelTable.put(new Integer( 1200 ), new JLabel(maxName) );
        throttlePedalSlider.setLabelTable(throttleLabelTable);
        throttlePedalSlider.setPaintLabels(true);
        throttlePedalSlider.setFont(font);
        throttlePedalSlider.addMouseListener(this);

        powerUnitLabel.setFont(font);

        powerLabel.setFont(font);

        sourceLabel.setFont(font);
        versionLabel.setFont(font);

        calcButton.setFont(font);
        calcButton.setActionCommand("calcButtonPressed");
        calcButton.addActionListener(this);

        pauseButton.setFont(font);
        pauseButton.setActionCommand("pauseButtonPressed");
        pauseButton.addActionListener(this);
        pauseButton.setEnabled(false);

        editTrackButton.setFont(font);
        editTrackButton.setActionCommand("editTrackButtonPressed");
        editTrackButton.addActionListener(this);
        editTrackButton.setEnabled(true);

        painter.car = loadImage(carImageName);
        painter.carTire = loadImage(carTireImageName);
        painter.carTireDriving = loadImage(carTireDrivingImageName);
        painter.guidePost = loadImage(guidePostImageName);
        painter.backGround = loadImage(backGroundImageName);
        painter.sun = loadImage(sunImageName);
        painter.moon = loadImage(moonImageName);
        painter.dove = loadImage(doveImageName);
        painter.doveWing = loadImage(doveWingImageName);
        painter.signalRed = loadImage(signalRedImageName);
        painter.signalYellow = loadImage(signalYellowImageName);
        painter.signalGreen = loadImage(signalGreenImageName);
        painter.gradientSign01=loadImage(gradientSign01Name);
        painter.gradientSign02=loadImage(gradientSign02Name);
        painter.signalX=painter.sceneWidth/2.0-250;
        painter.carSound=loadSound(carSoundWaveName);
        painter.doveSound=loadSound(doveSoundWaveName);

        painter.doveY=(Math.random() * painter.sceneHeight * 0.7);
        painter.doveStartX=painter.sceneWidth;
        doveCycle=0.0;
        painter.signalStartX=painter.sceneWidth/2.0-250.0;

        painter.guidePostX=0.0;
        painter.doveX=painter.doveStartX;
        painter.signalX=painter.signalStartX;
        painter.backGroundX=0.0;

        trackTextArea = new JTextArea("text");
        trackTextArea.setFont(fontFixed);
        trackTextArea.setEditable(true);

        //Put the editor pane in a scroll pane.
        textScrollPane = new JScrollPane(trackTextArea);
        textScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        textScrollPane.setPreferredSize(new Dimension((int)(painter.sceneWidth), (int)(painter.sceneHeight)));
        textScrollPane.setMinimumSize(new Dimension(100, 100));
        textScrollPane.setFont(fontFixed);
        textScrollPane.setVisible(false);

        //add components
        c.fill = GridBagConstraints.HORIZONTAL;
        c.ipadx = 10;
        c.insets = new Insets(5,5,5,5);  //padding

        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor=GridBagConstraints.CENTER;

        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(areaLabel, c);

        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(areaSpinner, c);

        c.gridx = 2;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(areaUnitLabel, c);

        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(cwLabel, c);

        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(cwSpinner, c);

        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(crLabel, c);

        c.gridx = 1;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(crSpinner, c);

        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(massLabel, c);

        c.gridx = 1;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(massSpinner, c);

        c.gridx = 2;
        c.gridy = 4;
        c.gridwidth=1;
        controlPanel.add(massUnitLabel, c);

        c.gridx = 0;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(powerLabel, c);

        c.gridx = 1;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(powerSpinner, c);

        c.gridx = 2;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(powerUnitLabel, c);

        c.gridx = 1;
        c.gridy = 9;
        c.gridwidth=2;
        controlPanel.add(calcButton, c);

        c.gridx = 0;
        c.gridy = 9;
        c.gridwidth=1;
        controlPanel.add(pauseButton, c);

        c.gridx = 0;
        c.gridy = 10;
        c.gridwidth=3;
        controlPanel.add(editTrackButton, c);

        c.gridx = 0;
        c.gridy = 11;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        controlPanel.add(sourceLabel, c);

        c.gridx = 0;
        c.gridy = 12;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        controlPanel.add(versionLabel, c);

        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=1;
        throttlePedalPanel.add(fixThrottleCheckBox, c);

        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth=1;
        throttlePedalPanel.add(throttlePedalLabel, c);

        c.gridx = 0;
        c.gridy = 3;
        throttlePedalPanel.add(throttlePedalSlider, c);

        checkBoxesControlPanel.add("Center", carPowerCheckBox);
        checkBoxesControlPanel.add("Center", airPowerCheckBox);
        checkBoxesControlPanel.add("Center", rollPowerCheckBox);
        checkBoxesControlPanel.add("Center", kineticPowerCheckBox);
        checkBoxesControlPanel.add("Center", potentialPowerCheckBox);


        //setup panels
        parentControlPanel.add(checkBoxesControlPanel);
        parentControlPanel.add(controlPanel);

        scenePanel.add("Center", painter);
        scenePanel.add("Center", textScrollPane);
        graphPanel.add("Center", graph01);
        graphPanel.add("Center", throttlePedalPanel);
        graphPanel.add("Center", parentControlPanel);

        mainPane.add(scenePanel);
        mainPane.add(graphPanel);
        getContentPane().add(mainPane);
        getRootPane().setDefaultButton(calcButton);

    }

    public JFormattedTextField getTextField(JSpinner spinner) {
        JComponent editor = spinner.getEditor();
        if (editor instanceof JSpinner.DefaultEditor) {
            return ((JSpinner.DefaultEditor)editor).getTextField();
        } else {
            System.err.println("Unexpected editor type: "  + spinner.getEditor().getClass() + " isn't a descendant of DefaultEditor");
            return null;
        }
    }


    public void mouseReleased(MouseEvent e) {
      if(!fixThrottleCheckBox.isSelected()){
       throttlePedalSlider.setValue(200);
      }
    }
    public void mouseExited(MouseEvent e) {

    }
    public void mouseEntered(MouseEvent e) {

    }
    public void mousePressed(MouseEvent e) {

    }
    public void mouseClicked(MouseEvent e) {

    }

    public int stringToInteger(String s){
      int i, minus=-1, digitNumber=0, result=0;
      char[] newString = new char[s.length()];
      String resultString="";

      for(i = 0; i < s.length(); i++){
        if(Character.isDigit(s.charAt(i))){
          newString[digitNumber]=s.charAt(i);
          digitNumber++;
        }
        else{
          if(digitNumber==0 && s.charAt(i)=='-'){
            newString[digitNumber]=s.charAt(i);
            digitNumber++;
          }
        }
      }
      if(digitNumber>0){
        resultString=new String(newString).substring(0, digitNumber);
        result=Integer.valueOf(resultString).intValue();
      }
      else{
        result=0;
      }
      return result;
    }

    // Listens to the buttons.
    public void actionPerformed(ActionEvent e) {
        int i, i2;

        if("editTrackButtonPressed".equals(e.getActionCommand())) {
          String newSection="", s="", sCut="", outString="";
          if(!editMode){
            painter.setVisible(false);
            textScrollPane.setVisible(true);
            trackTextArea.setText("");
            int lineBreak=0;
            for(i=0;i<100;i++){
              newSection="(";
              s = new DecimalFormat("#").format(i+1);
              newSection += "   ".substring( s.length() ) + s + ".) = ";
              if(i<painter.trackIMax){
                s = new DecimalFormat("#").format(painter.trackShapeDistance[i]);
                newSection += "     ".substring( s.length() ) + s + " / ";
                s =  new DecimalFormat("#").format(painter.trackShapeGradient[i]);
                newSection += "    ".substring( s.length() ) + s + " ; ";
              }
              else{
                newSection += "    0 /    0 ; ";
              }
              if (lineBreak==4){//Line break after 5 values
                lineBreak=-1;
                newSection += "\n";
              }
              outString+=newSection;
              lineBreak++;
            }
            trackTextArea.setText(outString);
            JOptionPane.showMessageDialog(null, editTrackHintName, editTrackName, JOptionPane.OK_CANCEL_OPTION);
            editMode=true;
            calcButton.setEnabled(false);
            editTrackButton.setText(editTrackDoneName);
            validate();
          }
          else{
            s = trackTextArea.getText();
            int lastEqualFound = 0, lastSlashFound = 0, foundIndex = 0, lastSemicolonFound = 0;
            double heightDiff=0.0;

            painter.trackDistance=0.0;
            while(lastEqualFound>-1 && lastSlashFound>-1 && lastSemicolonFound>-1 && foundIndex<100){
              lastEqualFound = s.indexOf("=", lastEqualFound + 1);
              lastSlashFound = s.indexOf("/", lastSlashFound + 1);
              lastSemicolonFound = s.indexOf(";", lastSemicolonFound + 1);
              if(lastSlashFound<lastEqualFound)lastSlashFound=lastEqualFound+1;
              if(lastSemicolonFound<lastSlashFound)lastSemicolonFound=lastSlashFound+1;
              if(lastSlashFound>s.length())lastSlashFound=-1;
              if(lastEqualFound>-1 && lastSlashFound>-1 && lastSemicolonFound>-1){
                sCut=s.substring(lastEqualFound+1, lastSlashFound) + "\0";
                painter.trackShapeDistance[foundIndex]=stringToInteger(sCut);
                if(painter.trackShapeDistance[foundIndex]<0){
                  painter.trackShapeDistance[foundIndex]=0;
                }
                if(painter.trackShapeDistance[foundIndex]>10000){
                  painter.trackShapeDistance[foundIndex]=10000;
                }
                sCut=s.substring(lastSlashFound+1, lastSemicolonFound) + "\0";
                painter.trackShapeGradient[foundIndex]=stringToInteger(sCut);
                if(painter.trackShapeGradient[foundIndex]<-20){
                  painter.trackShapeGradient[foundIndex]=-20;
                }
                if(painter.trackShapeGradient[foundIndex]>20){
                  painter.trackShapeGradient[foundIndex]=20;
                }
                painter.trackDistance+=painter.trackShapeDistance[foundIndex];
                if(foundIndex>0){
                  painter.trackShapeHeightSum[foundIndex]=Math.sin(Math.atan(painter.trackShapeGradient[foundIndex] / 100.0)) *
                                                          painter.trackShapeDistance[foundIndex];
                }
                else{
                  painter.trackShapeHeightSum[foundIndex]=Math.sin(Math.atan(painter.trackShapeGradient[0] / 100.0)) *
                                                          painter.trackShapeDistance[0];
                }
                if(painter.trackShapeDistance[foundIndex]==0){
                  break;
                }
                else{
                  heightDiff+=painter.trackShapeGradient[foundIndex]*painter.trackShapeDistance[foundIndex]/100.0;
                }
                foundIndex++;
              }
            }
            if(heightDiff>1 || heightDiff<-1){//Insert a track section to avoid height difference
              if(heightDiff<0){
                painter.trackShapeGradient[foundIndex]=20;
                painter.trackShapeDistance[foundIndex]=-(int)(100.0/20.0*heightDiff);
              }
              else{
                painter.trackShapeGradient[foundIndex]=-20;
                painter.trackShapeDistance[foundIndex]=(int)(100.0/20.0*heightDiff);
              }
              painter.trackDistance+=painter.trackShapeDistance[foundIndex];
              painter.trackShapeHeightSum[foundIndex]=painter.trackShapeHeightSum[foundIndex-1] +
                                                        Math.sin(Math.atan(painter.trackShapeGradient[foundIndex] / 100.0)) *
                                                        painter.trackShapeDistance[foundIndex];
              foundIndex++;
            }
            if(foundIndex==0){//No valid values found, one section is defined
              painter.trackShapeDistance[0]=10000;
              painter.trackShapeGradient[0]=0;
              painter.trackDistance=10000.0;
              painter.trackShapeHeightSum[0]=0.0;
              foundIndex=1;
            }
            painter.carDistance=0.0;
            painter.trackPosition=0.0;
            painter.trackI=0;
            painter.trackIMax=foundIndex;
            painter.setVisible(true);
            textScrollPane.setVisible(false);
            editTrackButton.setText(editTrackName);
            calcButton.setEnabled(true);
            editMode=false;
            validate();
          }
        }
        if("pauseButtonPressed".equals(e.getActionCommand())) {
          if(painter.isPause){
            painter.isPause=false;
            pauseButton.setText(pauseName);
          }
          else{
            painter.carSound.stop();
            painter.carSoundIsRunning=false;
            painter.isPause=true;
            pauseButton.setText(noPauseName);
            painter.refreshScene();
          }
        }
        if("calcButtonPressed".equals(e.getActionCommand())) {
          if(painter.doveSound != null){
            painter.doveSound.stop();
            painter.doveSoundIsRunning=false;
          }

          painter.isPause=false;
          pauseButton.setText(pauseName);

          if(isRunning){
            if(painter.carSound != null){
              painter.carSound.stop();
              painter.carSoundIsRunning=false;
            }
            calcButton.setText(startName);
            isRunning=false;
            painter.isRunning=false;
          }
          else{
            calcButton.setText(stopName);
            startTime=System.currentTimeMillis();
            simulationTime=0;
            simulationTimeOld=0;
            lapCounter=0;

            powerSpinner.setEnabled(false);
            cwSpinner.setEnabled(false);
            crSpinner.setEnabled(false);
            massSpinner.setEnabled(false);
            areaSpinner.setEnabled(false);
            editTrackButton.setEnabled(false);
            pauseButton.setEnabled(true);
            throttlePedalSlider.setValue(200);
            timer.setInitialDelay(timerPause);
            timer.start();

            painter.carSpeed=0.0;
            painter.carAngle=0.0;
            painter.trackI=0;
            painter.trackPosition=0.0;
            painter.carDistance=0.0;
            painter.carEnergy=0.0;
            painter.runTime=0.0;
            painter.trackHeight=0.0;
            isRunning=true;
            if(painter.doveX!=painter.doveStartX)painter.doveY=(Math.random() * painter.sceneHeight * 0.7);
            painter.doveStartX=painter.sceneWidth;
            doveCycle=0.0;
            painter.guidePostX=0.0;     //Return car to starting line
            painter.doveX=painter.doveStartX;
            painter.signalStartX=painter.sceneWidth/2.0-250.0;
            painter.signalX=painter.signalStartX;
            painter.backGroundX=0.0;
            painter.signalStatus=0;      //reset status of traffic light

            graph01.xOld=0;
            graph01.yOldPower=0;
            graph01.yOldAirPower=0;
            graph01.yOldRollPower=0;
            graph01.yOldKineticPower=0;
            graph01.yOldPotentialPower=0;
            graph01.yMax = powerModel.getNumber().doubleValue() * 1.2;
            graph01.paintNew=true;
            coordMulti = (double)graph01.graphHeight / graph01.yMax;
          }//if(!isRunning)
        }//if ("calcButtonPressed".equals(e.getActionCommand())) {
    }

    private class timerEvent implements ActionListener{
      public void actionPerformed(ActionEvent e){
        int i, i2;
        double timeDiff;

        simulationTimeOld=simulationTime;
        simulationTime=System.currentTimeMillis()-startTime;
        timeDiff=(double)(simulationTime-simulationTimeOld)/1000.0;
        if(timeDiff<=0.0 || timeDiff>1000.0){//This should normaly NOT happen !?!?
          //System.err.println("Unusual value for timeDiff: "  + timeDiff);
          timeDiff=0.000001;
        }

        if(painter.isPause){
          timeDiff=(double)(simulationTime-simulationTimeOld);
          if(timeDiff<=0.0 || timeDiff>1000.0){//This should normaly NOT happen !?!?
            //System.err.println("Unusual value for timeDiff: "  + timeDiff);
            timeDiff=0.000001;
          }
          startTime += timeDiff;
          simulationTime=System.currentTimeMillis()-startTime;
        }
        else{
          painter.runTime+=timeDiff;
          if(!isRunning){//End simulation
            timer.stop();
            powerSpinner.setEnabled(true);
            cwSpinner.setEnabled(true);
            crSpinner.setEnabled(true);
            massSpinner.setEnabled(true);
            areaSpinner.setEnabled(true);
            pauseButton.setEnabled(false);
            editTrackButton.setEnabled(true);
            isRunning=false;
            painter.carSpeed=0.0;
            painter.signalStatus=0;
            painter.isRunning=false;
            if(painter.carSound != null){
              painter.carSound.stop();
              painter.carSoundIsRunning=false;
            }
            if(painter.doveSound != null){
              painter.doveSound.stop();
            }
            calcButton.setText(startName);
            graph01.refreshScene();
          }//if(!isRunning){

          //car is 3meters long, picture of car is 195pixels wide: 195.0/3.0 is the number of pixels for 1meter.
          if(isRunning){
            if(painter.signalStatus>1){//Traffic light is green
              double carAcceleration;
              double carPower=0.0;
              double v1, v2;
              double throttleValue;
              double cw=0.0;
              double airAcceleration=0.0;
              double airPower=0.0;
              double carArea=1.51;
              double cr=0.0;
              double rollPower=0.0;
              double rollAcceleration=0.0;
              double kineticPower=0.0;
              double potentialPower=0.0;
              double potentialAcceleration=0.0;

              painter.carSpeedOld=painter.carSpeed;
              throttleValue=throttlePedalSlider.getValue();
              coordMulti = (double)graph01.graphHeight / graph01.yMax;
              cw=cwModel.getNumber().doubleValue();
              cr=crModel.getNumber().doubleValue();
              painter.carWeight=massModel.getNumber().doubleValue();
              carArea=areaModel.getNumber().doubleValue();

              rollAcceleration = painter.carWeight * 9.81 * cr / painter.carWeight;
              rollPower = painter.carWeight * 9.81 * cr * painter.carSpeed;
              airAcceleration = 0.5 * cw * carArea * 1.2041 * Math.pow(painter.carSpeed, 2.0) / painter.carWeight;
              airPower = 0.5 * cw * carArea * 1.2041 * Math.pow(painter.carSpeed, 2.0) * painter.carSpeed;
              potentialPower=Math.sin(painter.carAngle) * 9.81 * painter.carWeight * painter.carSpeed;

              if(throttleValue>200.0){//Car accelerates
                carPower = powerModel.getNumber().doubleValue() * (throttlePedalSlider.getValue() - 200.0);
                //Acceleration is calculated by given power and kinetic energy of the car
                v1=painter.carSpeed;
                v2=Math.sqrt(Math.pow(v1, 2.0) + 2 * carPower * timeDiff / painter.carWeight);
                carAcceleration = (v2 - v1) / timeDiff;
                painter.carEnergy += carPower * timeDiff;
              }
              else{//Car is breaking
                carAcceleration = -4.0 * (200.0-throttlePedalSlider.getValue())/200.0;
                carPower=0.0;
              }

              carAcceleration -= airAcceleration;
              carAcceleration -= rollAcceleration;
              carAcceleration -= Math.sin(painter.carAngle)*9.81;//Gradient force

              if(painter.carSpeed + carAcceleration * timeDiff>0.0){
                painter.carSpeed += carAcceleration * timeDiff;
              }
              else{
                painter.carSpeed = 0.0;
                carAcceleration=0.0;
              }

              kineticPower=0.5*painter.carWeight*(Math.pow(painter.carSpeed, 2.0) - Math.pow(painter.carSpeedOld, 2.0))/timeDiff;

              painter.carDistance += painter.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0);
              if(painter.carDistance > (lapCounter + 1) * painter.trackDistance){//A lap has been finished, simulation is paused
                painter.isPause=true;
                lapCounter++;
                pauseButton.setText(noPauseName);
              }
              painter.trackPosition += painter.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0);
              painter.signalX += (painter.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0))*195.0/3.0;

              while (painter.trackPosition>painter.trackShapeDistance[painter.trackI]){//Find out, which section of the track is active
                painter.trackPosition-=painter.trackShapeDistance[painter.trackI];
                painter.trackI++;
                if(painter.trackI>=painter.trackIMax)painter.trackI=0;
              }

              if(painter.trackI>0){
                painter.trackHeight=painter.trackShapeHeightSum[painter.trackI-1];
              }
              else{
                painter.trackHeight=0.0;
              }
              painter.trackHeight += painter.trackPosition*Math.sin(Math.atan(painter.trackShapeGradient[painter.trackI]/100.0));

              painter.guidePostX = painter.carDistance * 195.0 / 3.0;

              graph01.carDistance=(long)painter.carDistance;

              painter.doveX=(painter.carDistance -(simulationTime/1000.0)*doveSpeed)*195.0/3.0;
              doveCycle+=timeDiff*1000.0/350.0;
              painter.doveX+=painter.doveStartX;
              painter.doveWingHeight=(int) (59.0 * Math.sin(doveCycle));

              painter.backGroundX=painter.carDistance*195.0/3.0/300.0;
              while(painter.backGroundX>painter.sceneWidth)painter.backGroundX-=painter.sceneWidth;

              graph01.addLine((int)(painter.carDistance / graph01.metersPerGraphWidth * graph01.graphWidth),
                              (int)(carPower/1000.0 * coordMulti),
                              carPowerCheckBox.isSelected(),
                              (int)(airPower/1000.0 * coordMulti),
                              airPowerCheckBox.isSelected(),
                              (int)(rollPower/1000.0 * coordMulti),
                              rollPowerCheckBox.isSelected(),
                              (int)(kineticPower/1000.0 * coordMulti),
                              kineticPowerCheckBox.isSelected(),
                              (int)(potentialPower/1000.0 * coordMulti),
                              potentialPowerCheckBox.isSelected());

              if(isRunning){
                painter.isRunning=true;
              }
            }
            else{//traffic light turns from red to green
              if(simulationTime>1000){
                startTime=System.currentTimeMillis();
                simulationTime=0;
                painter.signalStatus++;
              }
            }
          }//if(isRunning)
          if(graph01.paintNew){//Graph is refreshed
            graph01.refreshScene();
          }
          painter.refreshScene();
        }//if(!(isPause))


      }//private class timerEvent implements ActionListener
    }//public void actionPerformed(ActionEvent e)
}



class ScenePainter extends Component {

    BufferedImage car = null, guidePost = null, carTire=null, carTireDriving = null, backGround = null;
    BufferedImage sun = null, moon = null, dove = null, doveWing = null;
    BufferedImage signalRed = null, signalYellow = null, signalGreen = null;
    BufferedImage gradientSign01 = null, gradientSign02 = null;
    BufferedImage bufferImage01=null;//buffer for the whole scene painting to avoid flickering
    int doveWingHeight=10;
    int sceneHeight = 200, sceneWidth = 900;
    int i;
    double guidePostX=0.0, guidePostY=0.0, backGroundX=0.0, doveX=-1000.0, doveY=10.0;
    double doveStartX=0.0, signalStartX=0.0;
    double signalX=0.0, signalY=0.0;
    int[] tireHub={0,0};  //makes the tires of the car jump while driving
    int tireMaxHub=1;
    Graphics2D bufferImage01Surface=null;
    AudioClip carSound=null, doveSound=null;
    double carEnergy=0.0;
    boolean carSoundIsRunning=false, doveSoundIsRunning=false;
    boolean isRunning=false;
    boolean carSoundStopped=true, doveSoundStopped=true;
    double carSpeed=0.0, carSpeedOld=0.0;
    int signalStatus=0;
    String hint01Name="", hint02Name="";
    int trackIMax=100, trackICount=0;
    int[] trackShapeDistance = new int[101];
    int[] trackShapeGradient = new int[101];
    double[] trackShapeHeightSum = new double[101];
    double trackDistance=10000.0, trackPosition=0.0, trackHeight=0.0;
    double carDistance=0.0;
    int trackI=0;
    double carAngle=0.0;
    double carWeight=765.0;
    double runTime=0.0;
    boolean isPause=false;
    String averageSpeedName="", trackHeightName="", energyInputName="";

    public Dimension getPreferredSize(){
        return new Dimension(sceneWidth, sceneHeight);
    }

    void refreshScene() {

      //play sound, if truck, dove, man etcetera reach middle of the scene (=position of car)
      if(doveSound!=null){
        if(doveX<sceneWidth/2.0 + 20.0 && doveX>sceneWidth/2.0 - 20.0){
          if (!doveSoundIsRunning && isRunning){
            doveSound.play();
            doveSoundIsRunning=true;
            doveSoundStopped=false;
          }
        }
        else{
          doveSoundIsRunning=false;
        }
        if((doveX<0.0 || doveX>sceneWidth) && !doveSoundStopped){
          doveSound.stop();
          doveSoundIsRunning=false;
          doveSoundStopped=true;
        }
      }
      while((int) guidePostX > (double) (sceneWidth) )guidePostX -= 195.0 * 50.0 / 3.0;

      //tires of car jump higher, the faster the car is driving
      tireMaxHub=1;
      if(carSpeed>1.0)tireMaxHub=1;
      if(carSpeed>10.0)tireMaxHub=3;
      if(carSpeed>30.0)tireMaxHub=4;
      if(carSpeed>50.0)tireMaxHub=5;
      repaint();
    }


    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension size = getSize();
        Composite origComposite;
        int sunX, sunY;
        boolean sunShine=false;
        Calendar cal = new GregorianCalendar();
        int secondsSinceMidnight = cal.get(Calendar.HOUR_OF_DAY) * 3600 + cal.get(Calendar.MINUTE) * 60 + cal.get(Calendar.SECOND);

        //create buffer to avoid flickering while scene is calculated and painted
        if(bufferImage01==null){
          bufferImage01 = (BufferedImage)createImage(sceneWidth, sceneHeight);
          bufferImage01Surface = bufferImage01.createGraphics();
          Font font = new Font("Arial", Font.PLAIN, 14);
          bufferImage01Surface.setFont(font);
        }

        origComposite = g2.getComposite();

        //Calculate Polygon of street
        int[] streetX = new int[110];
        int[] streetY = new int[110];

        carAngle = Math.atan(trackShapeGradient[trackI] / 100.0);

        streetX[1] = (int)(sceneWidth/2.0 + trackPosition * 195.0 / 3.0);
        streetY[1] = (int) (sceneHeight * 0.8 + trackShapeGradient[trackI] * (streetX[1]-sceneWidth/2.0) / 100.0 );

        int xTemp=streetX[1], yTemp=streetY[1];
        int trackINow=trackI;
        int shiftRight=0;

        while(xTemp < sceneWidth){//Find out how many sections of the track we have to move right, until we are out of the visible scene
          if(trackINow==0) trackINow=trackIMax;
          xTemp += (int)(trackShapeDistance[trackINow-1] * 195.0 / 3.0);
          yTemp += (int)(trackShapeGradient[trackINow-1] * trackShapeDistance[trackINow-1] / 100.0 * 195.0 / 3.0);
          trackINow--;
          shiftRight++;
        }

        trackICount=1;
        trackINow=trackI-shiftRight;
        while(trackINow<0) trackINow+=trackIMax;
        streetX[0]=xTemp;
        streetY[0]=yTemp;
        while(xTemp>0 && trackICount<100){//Move left at the sections of track ontil we are out of the visible range
          while(trackINow>=trackIMax) trackINow-=trackIMax;
          xTemp -= (int)(trackShapeDistance[trackINow] * 195.0 / 3.0);
          yTemp -= (int)(trackShapeGradient[trackINow] * trackShapeDistance[trackINow] / 100.0 * 195.0 / 3.0);
          streetX[trackICount]=xTemp;
          streetY[trackICount]=yTemp;
          if(guidePostX<streetX[trackICount-1] && guidePostX>streetX[trackICount]){
            guidePostY=streetY[trackICount] + (guidePostX-streetX[trackICount]) *
                          (streetY[trackICount-1]-streetY[trackICount]) /
                          (streetX[trackICount-1]-streetX[trackICount]);
          }
          if(signalX<streetX[trackICount-1] && signalX>streetX[trackICount]){
            signalY=streetY[trackICount] + (signalX-streetX[trackICount]) *
                          (streetY[trackICount-1]-streetY[trackICount]) /
                          (streetX[trackICount-1]-streetX[trackICount]);
          }
          trackINow++;
          trackICount++;
        }

        streetX[trackICount] = 0;
        streetY[trackICount] = sceneHeight;
        trackICount++;
        streetX[trackICount] = sceneWidth;
        streetY[trackICount] = sceneHeight;
        trackICount++;


        //calculates if sun (6:00 - 18:00) or moon (18:00 - 6:00) is shining
        //sunX: sun or moon are moving horizontal
        //suny: sun rising from 6:00 to 12:00 and declines from 12:00 - 18:00
        //      moon rising from 18:00 to 00:00 and declines from 00:00 - 6:00
        if(secondsSinceMidnight>6*3600 && secondsSinceMidnight<18*3600){//Sun shining
          secondsSinceMidnight-=6*3600;
          sunShine=true;
          sunX=(int)(sceneWidth * 0.1 + sceneWidth * 0.8 * secondsSinceMidnight / (3600.0 * 12.0));
          if(secondsSinceMidnight>6*3600){
            sunY=(int)( sceneHeight * 0.4 * ((secondsSinceMidnight - 3600.0 * 6.0) / (3600.0 * 6.0)));
          }
          else{
            sunY=(int)( sceneHeight * 0.4 * (1.0 - (secondsSinceMidnight / (3600.0 * 6.0))));
          }
        }
        else{//moon shining
          if(secondsSinceMidnight>18*3600){
            sunShine=false;
            secondsSinceMidnight-=12*3600;
          }
          if(secondsSinceMidnight>6*3600){
            sunX=(int)(sceneWidth * 0.5 - sceneWidth * 0.8 * (1.0 - secondsSinceMidnight / (3600.0 * 12.0)));
            sunY=(int)( sceneHeight * 0.4 * (1.0- (secondsSinceMidnight - 3600.0 * 6.0) / (3600.0 * 6.0)));
          }
          else{
            sunX=(int)(sceneWidth * 0.5 + sceneWidth * 0.8 * secondsSinceMidnight / (3600.0 * 12.0));
            sunY=(int)( sceneHeight * 0.4 * (secondsSinceMidnight / (3600.0 * 6.0)));
          }
        }
        //center of sun or moon sets position, not the left corner of the image
        if(sunShine && sun != null)sunX-=sun.getWidth(null)/2;
        if(!sunShine && moon != null)sunX-=moon.getWidth(null)/2;

        if(sunShine){//light blue background while sun is shining
          bufferImage01Surface.setBackground(new Color(223, 252, 255)); 
        }
        else{//black background during night
          bufferImage01Surface.setBackground(new Color(0, 0, 50)); 
        }
        bufferImage01Surface.clearRect(0, 0, sceneWidth, sceneHeight);

        if(sunShine){
          if(sun != null)bufferImage01Surface.drawImage(sun, sunX, sunY, 100, 100, this);
        }
        else{
          if(moon != null)bufferImage01Surface.drawImage(moon, sunX, sunY, 100, 100, this);
        }

        if(backGround!=null){
          bufferImage01Surface.drawImage(backGround, (int)backGroundX, 0, sceneWidth, sceneHeight, this);
          bufferImage01Surface.drawImage(backGround, (int)backGroundX-sceneWidth, 0, sceneWidth, sceneHeight, this);
        }
        if(guidePost != null)bufferImage01Surface.drawImage(guidePost, (int) (guidePostX), (int)(guidePostY-guidePost.getHeight(null)), this);


        double carTireX1=0.0, carTireY1=0.0, carTireX2=0.0, carTireY2=0.0, xRot=0.0, yRot=0.0;
        if(car != null){
          AffineTransform aTrans = AffineTransform.getRotateInstance(carAngle, sceneWidth / 2, sceneHeight / 2);
          bufferImage01Surface.setTransform(aTrans);
          bufferImage01Surface.drawImage(car,(sceneWidth-car.getWidth(null))/2, (int)(sceneHeight*0.8-car.getHeight(null)), this);
          aTrans = AffineTransform.getRotateInstance(Math.toRadians(0.0), car.getWidth() / 2, car.getHeight() / 2);
          bufferImage01Surface.setTransform(aTrans);
        }
        if(carTireDriving!=null){
            carTireX1=(sceneWidth-carTireDriving.getWidth(null))/2-68-sceneWidth/2.0;
            carTireY1=sceneHeight*0.4-carTireDriving.getHeight(null)-tireHub[0]+3.0;
            xRot = carTireX1 * Math.cos(-carAngle) + carTireY1 * Math.sin(-carAngle);
            yRot = -carTireX1 * Math.sin(-carAngle) + carTireY1* Math.cos(-carAngle);
            carTireX1=xRot+sceneWidth/2.0;
            carTireY1=yRot+sceneHeight*0.4;
            carTireX2=(sceneWidth-carTireDriving.getWidth(null))/2+55-sceneWidth/2.0;
            carTireY2=sceneHeight*0.4-carTireDriving.getHeight(null)-tireHub[1]+3.0;
            xRot = carTireX2 * Math.cos(-carAngle) + carTireY2 * Math.sin(-carAngle);
            yRot = -carTireX2 * Math.sin(-carAngle) + carTireY2* Math.cos(-carAngle);
            carTireX2=xRot+sceneWidth/2.0;
            carTireY2=yRot+sceneHeight*0.4;
        }
        if(isRunning && carSpeed>0.0){
          for(i=0;i<2;i++){
            tireHub[i]=(int)(Math.random() * (double)tireMaxHub);
          }
          if(carTireDriving != null){//let the tires jump while car is driving
            bufferImage01Surface.drawImage(carTireDriving,(int)(carTireX1), (int)(carTireY1), this);
            bufferImage01Surface.drawImage(carTireDriving,(int)(carTireX2), (int)(carTireY2), this);
          }
        }
        else{
          if(carTire != null){
            bufferImage01Surface.drawImage(carTire,(int)(carTireX1), (int)(carTireY1), this);
            bufferImage01Surface.drawImage(carTire,(int)(carTireX2), (int)(carTireY2), this);
          }
        }

        if(signalX<sceneWidth && signalX>0){//Signal is visible
          if(signalStatus==0){
            if(signalRed != null)bufferImage01Surface.drawImage(signalRed, 
                                                               (int)(signalX),
                                                               (int)(signalY+15.0-signalRed.getHeight(null)), this);
          }
          if(signalStatus==1){
            if(signalYellow != null)bufferImage01Surface.drawImage(signalYellow,
                                                                  (int)(signalX),
                                                                  (int)(signalY+15.0-signalRed.getHeight(null)), this);
          }
          if(signalStatus>1){
            if(signalGreen != null)bufferImage01Surface.drawImage(signalGreen,
                                                                 (int)(signalX),
                                                                 (int)(signalY+15.0-signalRed.getHeight(null)), this);
          }
        }
        if(signalX>sceneWidth){
          signalX -= trackDistance * 195.0 / 3.0;
        }

        bufferImage01Surface.fillPolygon(streetX, streetY, trackICount);//Paint street polygon

        if(carAngle!=0.0 && gradientSign01!=null && gradientSign02!=null){
          bufferImage01Surface.setColor(new Color(70, 70, 70));
          if(carAngle>0.0){
            bufferImage01Surface.drawImage(gradientSign02, 
                                           sceneWidth-gradientSign02.getWidth(null),
                                           sceneHeight-gradientSign02.getHeight(null), this);
            bufferImage01Surface.drawString(new DecimalFormat("00").format(trackShapeGradient[trackI]) + "%",
                                            sceneWidth- 50, sceneHeight-17);
          }
          else{
            bufferImage01Surface.drawImage(gradientSign01, 
                                           sceneWidth-gradientSign01.getWidth(null),
                                           sceneHeight-gradientSign01.getHeight(null), this);
            bufferImage01Surface.drawString(new DecimalFormat("00").format(-trackShapeGradient[trackI]) + "%",
                                            sceneWidth- 50, sceneHeight-17);
      }

        }

        if(dove != null && doveWing != null){
          if(doveX<sceneWidth && doveX>-dove.getWidth(null)){//paint dove
            bufferImage01Surface.drawImage(dove, (int)doveX, (int)doveY, this);
            bufferImage01Surface.drawImage(doveWing, (int)doveX+20, (int)doveY+15-doveWingHeight, doveWing.getWidth(null), doveWingHeight, this);
          }
        }

        if(!isPause){
          if(carSpeed==0.0 && carSoundIsRunning){
            carSound.stop();
            carSoundIsRunning=false;
          }
          if(carSpeed>0.0 && !carSoundIsRunning){
            carSound.loop();
            carSoundIsRunning=true;
          }
        }
        else{
          if(carSoundIsRunning){
            carSound.stop();
            carSoundIsRunning=false;
          }
        }

        //Give hints how to drive the car
        if(carSpeed==0.0 && signalStatus>1){
          bufferImage01Surface.setColor(new Color(255, 0, 0));
          bufferImage01Surface.drawString(hint01Name, 300, 20);
        }
        //Print speed, time and some other values
        bufferImage01Surface.setColor(new Color(255, 0, 100));
        bufferImage01Surface.drawString(energyInputName + ": " + new DecimalFormat("00.0000").format(carEnergy*0.000000277778) + "kWh    ", 1, 20);
        bufferImage01Surface.drawString(new DecimalFormat("000.00").format(carSpeed * 3.6) + "km/h = " + 
                                        new DecimalFormat("0.000").format(0.5 * carWeight * Math.pow(carSpeed, 2.0)*0.000000277778) + "kWh"
                                        , 1, 40);
        bufferImage01Surface.drawString(trackHeightName + ": " + new DecimalFormat("00.00").format(trackHeight) + "m = " +
                                        new DecimalFormat("0.000").format(trackHeight * 9.81 * carWeight*0.000000277778) + "kWh", 1, 60);
        if(isPause){
          bufferImage01Surface.drawString(averageSpeedName + ": " + new DecimalFormat("0.00").format(carDistance*3.6/runTime) + "km/h", 1, 80);
        }

        bufferImage01Surface.drawString(new DecimalFormat("000").format(runTime) + "sec", sceneWidth - 60, 20);

        bufferImage01Surface.setColor(new Color(0, 0, 0));
        bufferImage01Surface.drawRect(0, 0, sceneWidth-1, sceneHeight-1);

        //paint bufferImage01 and make it visible
        g2.drawImage(bufferImage01, 0, 0, this);

        g2.setComposite(origComposite);

    }
}

class GraphPainter extends Component {
    int graphWidth=350, graphHeight=220;
    Color[] colorUsed= {new Color(200, 0, 0),
                        new Color(0, 200, 0),
                        new Color(200, 0, 200),
                        new Color(0, 200, 200),
                        new Color(0, 0, 200),
                        new Color(200, 200, 0),
                        new Color(100, 100, 0)};
    BufferedImage bufferImage01 = null, bufferImage02 = null, bufferImage03 = null;
    Graphics2D bufferImage01Surface = null, bufferImage02Surface = null, bufferImage03Surface = null;
    int xOld, yOldPower, yOldAirPower, yOldRollPower, yOldKineticPower, yOldPotentialPower;
    long carDistance = 0;
    double yMax=17.0 * 1.2;
    String xString="", yString="";
    boolean paintNew=true;
    double metersPerGraphWidth=1000;

    public Dimension getPreferredSize(){
        return new Dimension(graphWidth + 50 + 50, graphHeight + 55);
    }

    void addLine(int xNew,
                 int yNewPower,
                 boolean paintPower,
                 int yNewAirPower,
                 boolean paintAirPower,
                 int yNewRollPower,
                 boolean paintRollPower,
                 int yNewKineticPower,
                 boolean paintKineticPower,
                 int yNewPotentialPower,
                 boolean paintPotentialPower){
      if(xNew != xOld){
        bufferImage02Surface.drawImage(bufferImage03, 0, 0, this);
        bufferImage03Surface.clearRect(0, 0, graphWidth+1, graphHeight);
        bufferImage03Surface.drawImage(bufferImage02,  - (xNew-xOld), 0, this);
        if(paintPower){
          bufferImage03Surface.setColor(colorUsed[0]);
          bufferImage03Surface.drawLine(graphWidth, graphHeight-yNewPower, graphWidth - (xNew-xOld), graphHeight-yOldPower);
        }
        if(paintAirPower){
          bufferImage03Surface.setColor(colorUsed[1]);
          bufferImage03Surface.drawLine(graphWidth, graphHeight-yNewAirPower, graphWidth - (xNew-xOld), graphHeight-yOldAirPower);
        }
        if(paintRollPower){
          bufferImage03Surface.setColor(colorUsed[2]);
          bufferImage03Surface.drawLine(graphWidth, graphHeight-yNewRollPower, graphWidth - (xNew-xOld), graphHeight-yOldRollPower);
        }
        if(paintKineticPower){
          if(yNewKineticPower>0){
            if(yOldKineticPower<0)yOldKineticPower=-yNewKineticPower;
            bufferImage03Surface.setColor(colorUsed[3]);
            bufferImage03Surface.drawLine(graphWidth, graphHeight-yNewKineticPower, graphWidth - (xNew-xOld), graphHeight-yOldKineticPower);
          }
          else{
            if(yOldKineticPower>0)yOldKineticPower=yNewKineticPower;
            bufferImage03Surface.setColor(colorUsed[4]);
            bufferImage03Surface.drawLine(graphWidth, graphHeight+yNewKineticPower, graphWidth - (xNew-xOld), graphHeight+yOldKineticPower);
          }
          //System.out.println("xOld=" + xOld + ", xNew=" + xNew + ", yNewKineticPower=" + yNewKineticPower + ", yOldKineticPower" + yOldKineticPower);
        }
        if(paintPotentialPower){
          if(yNewPotentialPower>0){
            if(yOldPotentialPower<0)yOldPotentialPower=-yNewPotentialPower;
            bufferImage03Surface.setColor(colorUsed[5]);
            bufferImage03Surface.drawLine(graphWidth, graphHeight-yNewPotentialPower, graphWidth - (xNew-xOld), graphHeight-yOldPotentialPower);
          }
          else{
            if(yOldPotentialPower>0)yOldPotentialPower=yNewPotentialPower;
            bufferImage03Surface.setColor(colorUsed[6]);
            bufferImage03Surface.drawLine(graphWidth, graphHeight+yNewPotentialPower, graphWidth - (xNew-xOld), graphHeight+yOldPotentialPower);
          }
        }
        yOldPower=yNewPower;
        yOldAirPower=yNewAirPower;
        yOldRollPower=yNewRollPower;
        yOldKineticPower=yNewKineticPower;
        yOldPotentialPower=yNewPotentialPower;
        repaint();
      }
      xOld=xNew;
    }

    void refreshScene(){
        repaint();
    }

    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension size = getSize();
        Composite origComposite;

        if(bufferImage01==null){
          bufferImage01 = (BufferedImage)createImage(graphWidth + 150, graphHeight + 55);
          bufferImage01Surface = bufferImage01.createGraphics();
          bufferImage01Surface.setBackground(new Color(238, 238, 238));
          bufferImage01Surface.clearRect(0, 0, graphWidth + 250, graphHeight + 55);
          Font font = new Font("Arial", Font.PLAIN, 12);
          bufferImage01Surface.setFont(font);
        }
        if(bufferImage02==null){
          bufferImage02 = (BufferedImage)createImage(graphWidth+1, graphHeight);
          bufferImage02Surface = bufferImage02.createGraphics();
          bufferImage02Surface.setBackground(new Color(238, 238, 238));
          bufferImage02Surface.clearRect(0, 0, graphWidth + 1, graphHeight);
        }
        if(bufferImage03==null){
          bufferImage03 = (BufferedImage)createImage(graphWidth+1, graphHeight);
          bufferImage03Surface = bufferImage03.createGraphics();
          bufferImage03Surface.setBackground(new Color(238, 238, 238));
          bufferImage03Surface.clearRect(0, 0, graphWidth + 1, graphHeight);
        }

        if(paintNew){
          bufferImage01Surface.clearRect(0, 0, graphWidth + 250, graphHeight + 55);

          bufferImage01Surface.setColor(Color.BLACK);
          //Y-Axis
          bufferImage01Surface.drawLine(50, 20, 50, graphHeight+30);
          bufferImage01Surface.drawLine(50, 20, 40, 30);
          bufferImage01Surface.drawLine(50, 20, 60, 30);
          bufferImage01Surface.drawLine(40, 20+(int)(graphHeight*0.25), 50, 20+(int)(graphHeight*0.25));
          bufferImage01Surface.drawLine(40, 20+(int)(graphHeight*0.5), 50, 20+(int)(graphHeight*0.5));
          bufferImage01Surface.drawLine(40, 20+(int)(graphHeight*0.75), 50, 20+(int)(graphHeight*0.75));
          //X-Axis
          bufferImage01Surface.drawLine(40, graphHeight+20, graphWidth+50, graphHeight+20);
          bufferImage01Surface.drawLine(graphWidth+40, graphHeight+10, graphWidth+50, graphHeight+20);
          bufferImage01Surface.drawLine(graphWidth+40, graphHeight+30, graphWidth+50, graphHeight+20);
          bufferImage01Surface.drawLine(50+(int)(graphWidth*0.25), graphHeight+20, 50+(int)(graphWidth*0.25), graphHeight+30);
          bufferImage01Surface.drawLine(50+(int)(graphWidth*0.5), graphHeight+20, 50+(int)(graphWidth*0.5), graphHeight+30);
          bufferImage01Surface.drawLine(50+(int)(graphWidth*0.75), graphHeight+20, 50+(int)(graphWidth*0.75), graphHeight+30);

          bufferImage01Surface.setColor(new Color(0, 0, 150));
          bufferImage01Surface.drawString(yString, 0, 10);
          bufferImage01Surface.drawString(xString, graphWidth+70, graphHeight+50);
          //labeling y-axis
          bufferImage01Surface.drawString(new DecimalFormat("0.0").format(yMax * 0.25), 0, (int)(graphHeight * 0.75 + 25.0));
          bufferImage01Surface.drawString(new DecimalFormat("0.0").format(yMax * 0.5), 0, (int)(graphHeight * 0.5 + 25.0));
          bufferImage01Surface.drawString(new DecimalFormat("0.0").format(yMax * 0.75), 0, (int)(graphHeight * 0.25 + 25.0));
          bufferImage01Surface.drawString(new DecimalFormat("0.0").format(yMax * 1.0), 0, (int)(graphHeight * 0.0 + 25.0));

          bufferImage02Surface.clearRect(0, 0, graphWidth, graphHeight);
          bufferImage03Surface.clearRect(0, 0, graphWidth, graphHeight);

          paintNew=false;
       }

        bufferImage01Surface.drawImage(bufferImage03, 51, 20, this);

        bufferImage01Surface.setColor(Color.BLACK);
        bufferImage01Surface.drawLine(graphWidth+40, graphHeight+10, graphWidth+50, graphHeight+20);
        bufferImage01Surface.drawLine(50, 20, 60, 30);

        //labeling x-axis
        bufferImage01Surface.clearRect(0, graphHeight + 31, graphWidth + 70, graphHeight + 55);
        bufferImage01Surface.setColor(new Color(0, 0, 150));
        bufferImage01Surface.drawString(new DecimalFormat("#").format(carDistance-metersPerGraphWidth*1.0),
                                       (int)(graphWidth * 0.0 + 30.0), graphHeight+50);
        bufferImage01Surface.drawString(new DecimalFormat("#").format(carDistance-metersPerGraphWidth*0.75),
                                       (int)(graphWidth * 0.25 + 30.0), graphHeight+50);
        bufferImage01Surface.drawString(new DecimalFormat("#").format(carDistance-metersPerGraphWidth*0.5),
                                       (int)(graphWidth * 0.5 + 30.0), graphHeight+50);
        bufferImage01Surface.drawString(new DecimalFormat("#").format(carDistance-metersPerGraphWidth*0.25),
                                       (int)(graphWidth * 0.75 + 30.0), graphHeight+50);
        bufferImage01Surface.drawString(new DecimalFormat("#").format(carDistance-metersPerGraphWidth*0.0),
                                       (int)(graphWidth * 1.0 + 30.0), graphHeight+50);

        g2.drawImage(bufferImage01, 0, 0, this);
    }
}

